Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import("resource://gre/modules/Timer.jsm");

const EXT_NAME = "toolbarize-menu-button";
const TOOLBAR_BUTTON_ID = EXT_NAME + "-toolbar-button";
const ICONS_URL = "chrome://" + EXT_NAME + "/content/images/icons/";
const DEFAULT_PREFS = {
    toolbarButtonIconTheme: 5,
    toolbarButtonPlaceId: "nav-bar",
    toolbarButtonPosition: 0,
    showToolbarButtonDropdownMark: true,
};

let Prefs = null;

function hideAppButtonForWindow(window) {
    let document = window.document;
    let appMenuButton = document.getElementById("appmenu-button") || document.getElementById("appmenu-toolbar-button");
    if (appMenuButton) {
        appMenuButton.originalDisplayStyle = appMenuButton.style.display;
        appMenuButton.style.display = "none";
        window.TabsOnTop.toggle();
        window.TabsOnTop.toggle();
    }
}

function restoreAppButtonForWindow(window) {
    let document = window.document;
    let appMenuButton = document.getElementById("appmenu-button") || document.getElementById("appmenu-toolbar-button");
    if (appMenuButton) {
        appMenuButton.style.display = appMenuButton.originalDisplayStyle;
        appMenuButton.originalDisplayStyle = undefined;
        window.TabsOnTop.toggle();
        window.TabsOnTop.toggle();
    }
}

function createToolbarButton(window, firstRun) {
    let document = window.document;
    let toolbarButton = document.createElement("toolbarbutton");
    toolbarButton.setAttribute("id", TOOLBAR_BUTTON_ID);
    toolbarButton.setAttribute("type", "menu");
    toolbarButton.setAttribute("class", "toolbarbutton-1 chromeclass-toolbar-additional");
    toolbarButton.style.listStyleImage = "url(" + ICONS_URL + Prefs.getValue("toolbarButtonIconTheme") + "/icon.png)";
    toolbarButton.style.MozBoxOrient = "inherit";
    
    let appMenuButton = document.getElementById("appmenu-button") || document.getElementById("appmenu-toolbar-button");
    if (appMenuButton)
        toolbarButton.setAttribute("tooltiptext", appMenuButton.getAttribute("tooltiptext"));
        toolbarButton.setAttribute("label", appMenuButton.getAttribute("tooltiptext"));
    
    let appMenuPopup = document.getElementById("appmenu-popup");
    if (appMenuPopup)
        toolbarButton.appendChild(appMenuPopup);
    
    let navigatorToolbox = document.getElementById("navigator-toolbox");
    navigatorToolbox.palette.appendChild(toolbarButton);
    
    let toolbarButtonPlaceId = Prefs.getValue("toolbarButtonPlaceId");
    if (toolbarButtonPlaceId != "") {
        if (firstRun) {
            let navBar = document.getElementById("nav-bar");
            navBar.insertItem(TOOLBAR_BUTTON_ID);
            let buttonsArray = navBar.currentSet.split(",");
            let buttonPosition = buttonsArray.indexOf(TOOLBAR_BUTTON_ID) + 1;
            Prefs.setValue("toolbarButtonPosition", buttonPosition);
            if (navBar.getAttribute("brighttext") == "true") {
                Prefs.setValue("toolbarButtonIconTheme", 2);
                toolbarButton.style.listStyleImage = "url(" + ICONS_URL + "2/icon.png)";
            }
            Prefs.save();
        } else {
            let toolbarButtonPosition = Prefs.getValue("toolbarButtonPosition");
            let someBar = document.getElementById(toolbarButtonPlaceId);
            if (someBar) {
                let buttonsArray = someBar.currentSet.split(",");
                let before = null;
                for (let i = toolbarButtonPosition - 1; i < buttonsArray.length; i++) {
                    before = document.getElementById(buttonsArray[i]);
                    if (before)
                        break;
                }
                
                someBar.insertItem(TOOLBAR_BUTTON_ID, before);
            }
        }
    }
    
    setTimeout(function() { 
        let toolbar = toolbarButton.parentNode;
        if (toolbar && toolbar.getAttribute("iconsize") == "large")
            toolbarButton.style.listStyleImage = "url(" + ICONS_URL + Prefs.getValue("toolbarButtonIconTheme") + "/icon_large.png)";
        
        if (!Prefs.getValue("showToolbarButtonDropdownMark"))
            removeDropdownMark(toolbarButton);
    }, 0);
    
    window.addEventListener("beforecustomization", beforeCustomization, false);
    window.addEventListener("aftercustomization", afterCustomization, false);
}

function removeDropdownMark(toolbarButton) {
    let document = toolbarButton.ownerDocument;
    let toolbarButtonMenuDropmarker = document.getAnonymousElementByAttribute(toolbarButton, "anonid", "dropmarker");
    if (toolbarButtonMenuDropmarker) {
        let toolbar = toolbarButton.parentNode;
        if (toolbar && toolbar.getAttribute("iconsize") == "small" && !toolbar.getAttribute("lockiconsize")) {
            let topToolbar = toolbarButton.parentNode.parentNode;
            if (topToolbar && topToolbar.getAttribute("iconsize") == "large") {
                toolbarButtonMenuDropmarker.style.display = "inline";
                toolbarButtonMenuDropmarker.style.visibility = "hidden";
                let toolbarButtonIcon = document.getAnonymousElementByAttribute(toolbarButton, "class", "toolbarbutton-icon");
                if (toolbarButtonIcon)
                    toolbarButtonIcon.style.MozPaddingEnd = "7px";
            } else
                toolbarButtonMenuDropmarker.style.display = "none";
        } else
            toolbarButtonMenuDropmarker.style.display = "none";
    }
}

function removeToolbarButton(window) {
    window.removeEventListener("beforecustomization", beforeCustomization, false);
    window.removeEventListener("aftercustomization", afterCustomization, false);
    
    let document = window.document;
    let toolbarButton = document.getElementById(TOOLBAR_BUTTON_ID);
    if (toolbarButton)
        toolbarButton.parentNode.removeChild(toolbarButton);
    
    let navigatorToolbox = document.getElementById("navigator-toolbox");
    for (let nodeIndex in navigatorToolbox.palette.childNodes) {
        let childNode = navigatorToolbox.palette.childNodes[nodeIndex];
        if (childNode && childNode.id == TOOLBAR_BUTTON_ID)
            navigatorToolbox.palette.removeChild(childNode);
    }
}

function beforeCustomization(event) {
    let navigatorToolbox = event.target;
    let window = navigatorToolbox.ownerDocument.defaultView;
    fixAppMenuPopup(window);
}

function afterCustomization(event) {
    let navigatorToolbox = event.target;
    let window = navigatorToolbox.ownerDocument.defaultView;
    let newToolbarButtonPlaceId = "";
    let newToolbarButtonPosition = 0;
    let toolbarButton = window.document.getElementById(TOOLBAR_BUTTON_ID);
    if (toolbarButton) {
        let buttonParent = toolbarButton.parentNode;
        if (buttonParent.currentSet && buttonParent.id) {
            newToolbarButtonPlaceId = buttonParent.id;
            let buttonsArray = buttonParent.currentSet.split(",");
            newToolbarButtonPosition = buttonsArray.indexOf(TOOLBAR_BUTTON_ID) + 1;
        }
        
        setTimeout(function() {
            let toolbar = toolbarButton.parentNode;
            if (toolbar && toolbar.getAttribute("iconsize") == "large")
                toolbarButton.style.listStyleImage = "url(" + ICONS_URL + Prefs.getValue("toolbarButtonIconTheme") + "/icon_large.png)";
            else
                toolbarButton.style.listStyleImage = "url(" + ICONS_URL + Prefs.getValue("toolbarButtonIconTheme") + "/icon.png)";
            
            if (!Prefs.getValue("showToolbarButtonDropdownMark"))
                removeDropdownMark(toolbarButton);
        }, 0);
        
        let appMenuPopup = window.document.getElementById("appmenu-popup");
        if (appMenuPopup)
            toolbarButton.appendChild(appMenuPopup);
    }
    
    Prefs.setValue("toolbarButtonPlaceId", newToolbarButtonPlaceId);
    Prefs.setValue("toolbarButtonPosition", newToolbarButtonPosition);
    Prefs.save();
};

function fixAppMenuPopup(window) {
    let document = window.document;
    let appMenuPopup = document.getElementById("appmenu-popup");
    if (appMenuPopup) {
        let appMenuButton = document.getElementById("appmenu-button") || document.getElementById("appmenu-toolbar-button");
        appMenuButton.appendChild(appMenuPopup);
    }
}

let windowListener = {
    onOpenWindow: function(nsIObj) {
        let window = nsIObj.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                           .getInterface(Components.interfaces.nsIDOMWindow);
        window.addEventListener("load", function() {
            window.removeEventListener("load", arguments.callee, false);
            if (window.document.documentElement.getAttribute("windowtype") == "navigator:browser") {
                hideAppButtonForWindow(window);
                createToolbarButton(window);
            }
        });
    }
};

function initWindows(firstRun) {
    let windowsEnumerator = Services.wm.getEnumerator("navigator:browser");
    while (windowsEnumerator.hasMoreElements()) {
        let window = windowsEnumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindow);
        hideAppButtonForWindow(window);
        createToolbarButton(window, firstRun);
    }
    
    Services.wm.addListener(windowListener);
}

function cleanupWindows() {
    Services.wm.removeListener(windowListener);
    
    let windowsEnumerator = Services.wm.getEnumerator("navigator:browser");
    while (windowsEnumerator.hasMoreElements()) {
        let window = windowsEnumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindow);
        fixAppMenuPopup(window);
        restoreAppButtonForWindow(window);
        removeToolbarButton(window);
    }
}

function startup(data, reason) {
    let Imports = {};
    Components.utils.import("chrome://" + EXT_NAME + "/content/modules/preflib.js", Imports);
    
    Prefs = new Imports.Prefs(DEFAULT_PREFS, EXT_NAME, true, function() {
        cleanupWindows();
        initWindows();
    });
    
    Prefs.init();
    initWindows(reason == ADDON_INSTALL);
}

function shutdown(data, reason) {
    if (reason != APP_SHUTDOWN) {
        Prefs.cleanup();
        cleanupWindows();
    }
}

function install(data, reason) {}

function uninstall(data, reason) {}