Components.utils.import("resource://gre/modules/Services.jsm");

let EXPORTED_SYMBOLS = ["Prefs"];
let Prefs = function(DEFAULT_PREFS, EXT_NAME, enableSync, onApplyCallbackMethod) {
    this.defaultPrefs = DEFAULT_PREFS;    
    this.currentPrefs = {};
    
    this.prefsBranch = Services.prefs.getBranch("extensions." + EXT_NAME + ".");
    this.syncPrefsBranch = Services.prefs.getBranch("services.sync.prefs.sync.extensions." + EXT_NAME + ".");
    
    this.init = function() {
        for (let prefName in this.defaultPrefs) {
            let prefNotExists = !this.prefsBranch.getPrefType(prefName);
            let prefValue = this.defaultPrefs[prefName];
            let prefType = typeof prefValue;
            switch (prefType) {
                case "string":
                    if (prefNotExists)
                        this.prefsBranch.setCharPref(prefName, prefValue);
                    this.currentPrefs[prefName] = this.prefsBranch.getCharPref(prefName);
                    break;
                case "number":
                    if (prefNotExists)
                        this.prefsBranch.setIntPref(prefName, prefValue);
                    this.currentPrefs[prefName] = this.prefsBranch.getIntPref(prefName);
                    break;
                case "boolean":
                    if (prefNotExists)
                        this.prefsBranch.setBoolPref(prefName, prefValue);
                    this.currentPrefs[prefName] = this.prefsBranch.getBoolPref(prefName);
                    break;
            }
            
            if (enableSync) {
                let syncPrefNotExists = !this.syncPrefsBranch.getPrefType(prefName);
                let syncPrefValue = syncPrefNotExists ? true : this.syncPrefsBranch.getBoolPref(prefName);
                this.syncPrefsBranch.setBoolPref(prefName, syncPrefValue);
            }
        }
        
        Services.obs.addObserver(this.onOpen, EXT_NAME + "-prefs-open", false);
        Services.obs.addObserver(this.onReset, EXT_NAME + "-prefs-reset", false);
        Services.obs.addObserver(this.onApply, EXT_NAME + "-prefs-apply", false);
    };
    
    this.cleanup = function() {
        Services.obs.removeObserver(this.onOpen, EXT_NAME + "-prefs-open");
        Services.obs.removeObserver(this.onReset, EXT_NAME + "-prefs-reset");
        Services.obs.removeObserver(this.onApply, EXT_NAME + "-prefs-apply");
    };
    
    this.save = function() {
        for (let prefName in this.currentPrefs) {
            let prefValue = this.currentPrefs[prefName];
            let prefType = typeof prefValue;
            switch (prefType) {
                case "string":
                    this.prefsBranch.setCharPref(prefName, prefValue);
                    break;
                case "number":
                    this.prefsBranch.setIntPref(prefName, prefValue);
                    break;
                case "boolean":
                    this.prefsBranch.setBoolPref(prefName, prefValue);
                    break;
            }
        }   
    };
    
    this.getValue = function(prefName) {
        return this.currentPrefs[prefName];
    };
    
    this.setValue = function(prefName, prefValue) {
        this.currentPrefs[prefName] = prefValue;
    };
    
    this.feedPrefsWindow = function(window, feedDefaults) {
        for (let prefName in this.defaultPrefs) {
            let prefControl = window.document.getElementById(prefName);
            if (prefControl) {
                let prefValue = feedDefaults ? this.defaultPrefs[prefName] : this.currentPrefs[prefName];
                if (prefControl.type == "number") {
                    prefControl.valueNumber = prefValue;
                } else if (prefControl.tagName == "checkbox") {
                    prefControl.checked = prefValue;
                } else {
                    prefControl.value = prefValue;
                }
            }
        }
    };
    
    this.saveFromPrefsWindow = function(window) {
        for (let prefName in this.currentPrefs) {
            let prefControl = window.document.getElementById(prefName);
            if (prefControl) {
                let prefValue = null;
                if (prefControl.type == "number" || prefControl.tagName == "menulist") {
                    prefValue = parseInt(prefControl.value);
                } else if (prefControl.tagName == "checkbox") {
                    prefValue = prefControl.checked;
                } else {
                    prefValue = prefControl.value;
                }
                
                if (prefValue != null)
                    this.currentPrefs[prefName] = prefValue;
            }
        }
        
        this.save();
    };
    
    this.onOpen = {
        Prefs: this,
        feedPrefsWindow: this.feedPrefsWindow,
        observe: function(aSubject, aTopic, aData) {
            this.feedPrefsWindow.call(this.Prefs, aSubject);
        }
    };
    
    this.onReset = {
        Prefs: this,
        feedPrefsWindow: this.feedPrefsWindow,
        observe: function(aSubject, aTopic, aData) {
            this.feedPrefsWindow.call(this.Prefs, aSubject, true);
        }
    };
    
    this.onApply = {
        Prefs: this,
        saveFromPrefsWindow: this.saveFromPrefsWindow,
        observe: function(aSubject, aTopic, aData) {
            this.saveFromPrefsWindow.call(this.Prefs, aSubject);
            onApplyCallbackMethod();
        }
    };
};